@@ -23,9 +23,11 @@
 #include <stdlib.h>
 
 #include "pango-renderer.h"
+#include "pango-layout-private.h"
 
 #define N_RENDER_PARTS 4
 
+#define PANGO_IS_RENDERER_FAST(renderer) (renderer != NULL)
 #define IS_VALID_PART(part) ((guint)part < N_RENDER_PARTS)
 
 typedef struct _LineState LineState;
@@ -175,7 +177,7 @@ pango_renderer_draw_layout (PangoRenderer    *renderer,
       PangoLayoutLine *line;
       int              baseline;
       
-      line = pango_layout_iter_get_line (iter);
+      line = _pango_layout_iter_get_line_readonly (iter);
       
       pango_layout_iter_get_line_extents (iter, NULL, &logical_rect);
       baseline = pango_layout_iter_get_baseline (iter);
@@ -452,7 +454,7 @@ pango_renderer_draw_layout_line (PangoRenderer    *renderer,
   gboolean got_overall = FALSE;
   PangoRectangle overall_rect;
   
-  g_return_if_fail (PANGO_IS_RENDERER (renderer));
+  g_return_if_fail (PANGO_IS_RENDERER_FAST (renderer));
 
   /* We only change the matrix if the renderer isn't already
    * active.
@@ -503,8 +505,9 @@ pango_renderer_draw_layout_line (PangoRenderer    *renderer,
 	      ink = &ink_rect;
 	      logical = &logical_rect;
 	    }
-	  pango_glyph_string_extents (run->glyphs, run->item->analysis.font,
-				      ink, logical);
+	  if (G_UNLIKELY (ink || logical))
+	    pango_glyph_string_extents (run->glyphs, run->item->analysis.font,
+					ink, logical);
 	  if (logical)
 	    glyph_string_width = logical_rect.width;
 	  else
@@ -604,7 +607,7 @@ pango_renderer_draw_glyphs (PangoRenderer    *renderer,
 			    int               x,
 			    int               y)
 {
-  g_return_if_fail (PANGO_IS_RENDERER (renderer));
+  g_return_if_fail (PANGO_IS_RENDERER_FAST (renderer));
   
   pango_renderer_activate (renderer);
 
@@ -664,7 +667,7 @@ pango_renderer_draw_rectangle (PangoRenderer   *renderer,
 			       int              width,
 			       int              height)
 {
-  g_return_if_fail (PANGO_IS_RENDERER (renderer));
+  g_return_if_fail (PANGO_IS_RENDERER_FAST (renderer));
   g_return_if_fail (IS_VALID_PART (part));
   g_return_if_fail (renderer->active_count > 0);
   
@@ -804,7 +807,7 @@ pango_renderer_draw_error_underline (PangoRenderer *renderer,
 				     int            width,
 				     int            height)
 {
-  g_return_if_fail (PANGO_IS_RENDERER (renderer));
+  g_return_if_fail (PANGO_IS_RENDERER_FAST (renderer));
   g_return_if_fail (renderer->active_count > 0);
   
   PANGO_RENDERER_GET_CLASS (renderer)->draw_error_underline (renderer, x, y, width, height);
@@ -952,7 +955,7 @@ pango_renderer_draw_trapezoid (PangoRenderer  *renderer,
 			       double          x12,
 			       double          x22)
 {
-  g_return_if_fail (PANGO_IS_RENDERER (renderer));
+  g_return_if_fail (PANGO_IS_RENDERER_FAST (renderer));
   g_return_if_fail (renderer->active_count > 0);
   
   if (PANGO_RENDERER_GET_CLASS (renderer)->draw_trapezoid)
@@ -980,7 +983,7 @@ pango_renderer_draw_glyph (PangoRenderer *renderer,
 			   double         x,
 			   double         y)
 {
-  g_return_if_fail (PANGO_IS_RENDERER (renderer));
+  g_return_if_fail (PANGO_IS_RENDERER_FAST (renderer));
   g_return_if_fail (renderer->active_count > 0);
 	  
   if (glyph == PANGO_GLYPH_EMPTY) /* glyph PANGO_GLYPH_EMPTY never renders */
@@ -1007,7 +1010,7 @@ pango_renderer_draw_glyph (PangoRenderer *renderer,
 void
 pango_renderer_activate (PangoRenderer *renderer)
 {
-  g_return_if_fail (PANGO_IS_RENDERER (renderer));
+  g_return_if_fail (PANGO_IS_RENDERER_FAST (renderer));
 
   renderer->active_count++;
   if (renderer->active_count == 1)
@@ -1029,7 +1032,7 @@ pango_renderer_activate (PangoRenderer *renderer)
 void
 pango_renderer_deactivate (PangoRenderer *renderer)
 {
-  g_return_if_fail (PANGO_IS_RENDERER (renderer));
+  g_return_if_fail (PANGO_IS_RENDERER_FAST (renderer));
   g_return_if_fail (renderer->active_count > 0);
   
   if (renderer->active_count == 1)
@@ -1055,7 +1058,7 @@ pango_renderer_set_color (PangoRenderer    *renderer,
 			  PangoRenderPart   part,
 			  const PangoColor *color)
 {
-  g_return_if_fail (PANGO_IS_RENDERER (renderer));
+  g_return_if_fail (PANGO_IS_RENDERER_FAST (renderer));
   g_return_if_fail (IS_VALID_PART (part));
 
   if ((!color && !renderer->priv->color_set[part]) ||
@@ -1095,7 +1098,7 @@ PangoColor *
 pango_renderer_get_color (PangoRenderer   *renderer,
 			  PangoRenderPart  part)
 {
-  g_return_val_if_fail (PANGO_IS_RENDERER (renderer), NULL);
+  g_return_val_if_fail (PANGO_IS_RENDERER_FAST (renderer), NULL);
   g_return_val_if_fail (IS_VALID_PART (part), NULL);
   
   if (renderer->priv->color_set[part])
@@ -1129,7 +1132,7 @@ void
 pango_renderer_part_changed (PangoRenderer    *renderer,
 			     PangoRenderPart   part)
 {
-  g_return_if_fail (PANGO_IS_RENDERER (renderer));
+  g_return_if_fail (PANGO_IS_RENDERER_FAST (renderer));
   g_return_if_fail (IS_VALID_PART (part));
   g_return_if_fail (renderer->active_count > 0);
   
@@ -1152,7 +1155,7 @@ static void
 pango_renderer_prepare_run (PangoRenderer  *renderer,
 			    PangoLayoutRun *run)
 {
-  g_return_if_fail (PANGO_IS_RENDERER (renderer));
+  g_return_if_fail (PANGO_IS_RENDERER_FAST (renderer));
   
   PANGO_RENDERER_GET_CLASS (renderer)->prepare_run (renderer, run);
 }
@@ -1231,7 +1234,7 @@ void
 pango_renderer_set_matrix (PangoRenderer     *renderer,
 			   const PangoMatrix *matrix)
 {
-  g_return_if_fail (PANGO_IS_RENDERER (renderer));
+  g_return_if_fail (PANGO_IS_RENDERER_FAST (renderer));
 
   if (renderer->matrix)
     pango_matrix_free (renderer->matrix);
