@@ -20,14 +20,17 @@
  */
 
 /* Size of fontset cache */
-#define FONTSET_CACHE_SIZE 16
+#define FONTSET_CACHE_SIZE 64
 
+#include <config.h>
 #include <math.h>
 
 #include "pango-context.h"
 #include "pangofc-fontmap.h"
 #include "pangofc-private.h"
+#include "pango-impl-utils.h"
 #include "modules.h"
+#include "pango-enum-types.h"
 
 typedef struct _PangoFcCoverageKey  PangoFcCoverageKey;
 typedef struct _PangoFcFace         PangoFcFace;
@@ -159,7 +162,7 @@ static void
 pango_fc_font_map_init (PangoFcFontMap *fcfontmap)
 {
   static gboolean registered_modules = FALSE;
-  PangoFcFontMapPrivate *priv = fcfontmap->priv;
+  PangoFcFontMapPrivate *priv;
 
   priv = fcfontmap->priv = G_TYPE_INSTANCE_GET_PRIVATE (fcfontmap,
 							PANGO_TYPE_FC_FONT_MAP,
@@ -209,6 +212,17 @@ pango_fc_font_map_class_init (PangoFcFontMapClass *class)
   g_type_class_add_private (object_class, sizeof (PangoFcFontMapPrivate));
 }
 
+static gpointer
+get_gravity_class (void)
+{
+  static GEnumClass *class = NULL;
+
+  if (G_UNLIKELY (!class))
+    class = g_type_class_ref (PANGO_TYPE_GRAVITY);
+
+  return class;
+}
+
 static guint
 pango_fc_pattern_hash (FcPattern *pattern)
 {
@@ -304,6 +318,7 @@ fontset_hash_key_equal (const FontsetHashKey *key_a,
 {
   if (key_a->size == key_b->size &&
       pango_font_description_equal (key_a->desc, key_b->desc) &&
+      key_a->language == key_b->language &&
       key_a->matrix.xx == key_b->matrix.xx &&
       key_a->matrix.xy == key_b->matrix.xy &&
       key_a->matrix.yx == key_b->matrix.yx &&
@@ -350,13 +365,13 @@ fontset_hash_key_free (FontsetHashKey *key)
     PANGO_FC_FONT_MAP_GET_CLASS (key->fontmap)->context_key_free (key->fontmap,
 								  key->context_key);
   
-  g_free (key);
+  g_slice_free (FontsetHashKey, key);
 }
 
 static FontsetHashKey *
 fontset_hash_key_copy (FontsetHashKey *old)
 {
-  FontsetHashKey *key = g_new (FontsetHashKey, 1);
+  FontsetHashKey *key = g_slice_new (FontsetHashKey);
   
   key->fontmap = old->fontmap;
   key->matrix = old->matrix;
@@ -417,13 +432,13 @@ font_hash_key_free (FontHashKey *key)
     PANGO_FC_FONT_MAP_GET_CLASS (key->fontmap)->context_key_free (key->fontmap,
 								  key->context_key);
   
-  g_free (key);
+  g_slice_free (FontHashKey, key);
 }
 
 static FontHashKey *
 font_hash_key_copy (FontHashKey *old)
 {
-  FontHashKey *key = g_new (FontHashKey, 1);
+  FontHashKey *key = g_slice_new (FontHashKey);
   
   key->fontmap = old->fontmap;
   key->matrix = old->matrix;
@@ -463,7 +478,7 @@ pango_fc_font_map_add_decoder_find_func (PangoFcFontMap        *fcfontmap,
   PangoFcFontMapPrivate *priv = fcfontmap->priv;
   PangoFcFindFuncInfo *info;
 
-  info = g_new (PangoFcFindFuncInfo, 1);
+  info = g_slice_new (PangoFcFindFuncInfo);
 
   info->findfunc = findfunc;
   info->user_data = user_data;
@@ -482,6 +497,9 @@ pango_fc_font_map_finalize (GObject *object)
   g_queue_free (priv->fontset_cache);
   g_hash_table_destroy (priv->coverage_hash);
 
+  if (priv->fontset_hash)
+    g_hash_table_destroy (priv->fontset_hash);
+
   if (priv->font_hash)
     g_hash_table_destroy (priv->font_hash);
 
@@ -495,7 +513,7 @@ pango_fc_font_map_finalize (GObject *object)
       if (info->dnotify)
 	info->dnotify (info->user_data);
 
-      g_free (info);
+      g_slice_free (PangoFcFindFuncInfo, info);
       priv->findfuncs = g_slist_delete_link (priv->findfuncs, priv->findfuncs);
     }
 
@@ -552,7 +570,7 @@ pango_fc_font_map_add (PangoFcFontMap *fcfontmap,
   key.pattern = fcfont->font_pattern;
   
   key_copy = font_hash_key_copy (&key);
-  fcfont->context_key = key_copy->context_key;
+  _pango_fc_font_set_context_key (fcfont, key_copy->context_key);
   fcfont->matrix = key.matrix;
   
   g_hash_table_insert (priv->font_hash, key_copy, fcfont);
@@ -583,11 +601,11 @@ _pango_fc_font_map_remove (PangoFcFontMap *fcfontmap,
   key.fontmap = fcfontmap;
   key.matrix = fcfont->matrix;
   key.pattern = fcfont->font_pattern;
-  key.context_key = fcfont->context_key;
+  key.context_key = _pango_fc_font_get_context_key (fcfont);
   
   g_hash_table_remove (priv->font_hash, &key);
   fcfont->fontmap = NULL;
-  fcfont->context_key = NULL;
+  _pango_fc_font_set_context_key (fcfont, NULL);
   g_object_unref (fcfontmap);
 }
 
@@ -788,6 +806,8 @@ pango_fc_make_pattern (const  PangoFontDescription *description,
   FcPattern *pattern;
   int slant;
   int weight;
+  PangoGravity gravity;
+  FcBool vertical;
   char **families;
   int i;
 #ifdef FC_WIDTH
@@ -800,6 +820,20 @@ pango_fc_make_pattern (const  PangoFontDescription *description,
   width = pango_fc_convert_width_to_fc (pango_font_description_get_stretch (description));
 #endif
 
+  gravity = pango_font_description_get_gravity (description);
+  switch (gravity)
+    {
+      case PANGO_GRAVITY_SOUTH:
+      case PANGO_GRAVITY_NORTH:
+      default:
+        vertical = FcFalse;
+	break;
+      case PANGO_GRAVITY_EAST:
+      case PANGO_GRAVITY_WEST:
+        vertical = FcTrue;
+        break;
+    }
+
   /* The reason for passing in FC_SIZE as well as FC_PIXEL_SIZE is
    * to work around a bug in libgnomeprint where it doesn't look
    * for FC_PIXEL_SIZE. See http://bugzilla.gnome.org/show_bug.cgi?id=169020
@@ -813,6 +847,9 @@ pango_fc_make_pattern (const  PangoFontDescription *description,
 			    FC_SLANT,  FcTypeInteger, slant,
 #ifdef FC_WIDTH
 			    FC_WIDTH,  FcTypeInteger, width,
+#endif
+#ifdef FC_VERTICAL_LAYOUT
+			    FC_VERTICAL_LAYOUT,  FcTypeBool, vertical,
 #endif
  			    FC_SIZE,  FcTypeDouble,  pixel_size * (72. / dpi),
  			    FC_PIXEL_SIZE,  FcTypeDouble,  pixel_size,
@@ -827,6 +864,12 @@ pango_fc_make_pattern (const  PangoFontDescription *description,
 
   if (language)
     FcPatternAddString (pattern, FC_LANG, (FcChar8 *) pango_language_to_string (language));
+
+  if (gravity != PANGO_GRAVITY_SOUTH)
+    {
+      GEnumValue *value = g_enum_get_value (get_gravity_class (), gravity);
+      FcPatternAddString (pattern, PANGO_FC_GRAVITY, value->value_nick);
+    }
   
   return pattern;
 }
@@ -844,12 +887,6 @@ pango_fc_font_map_new_font (PangoFontMap               *fontmap,
   PangoFcFont *fcfont;
   GSList *l;
 
-  /* Returning NULL here actually violates a contract
-   * that loading load_font() will never return NULL.
-   * We probably should actually create a dummy
-   * font that doesn't draw anything and has empty
-   * metrics.
-   */
   if (priv->closed)
     return NULL;
   
@@ -898,6 +935,9 @@ pango_fc_font_map_new_font (PangoFontMap               *fontmap,
 	FcPatternDestroy (pattern);
     }
 
+  if (!fcfont)
+    return NULL;
+
   pango_fc_font_map_add (fcfontmap, context, fcfont);
 
   /*
@@ -1007,44 +1047,6 @@ get_unscaled_size (PangoFcFontMap             *fcfontmap,
     }
 }
 
-/*
- * Based on cairo-matrix.c:_cairo_matrix_compute_scale_factors()
- *
- * Copyright 2005, Keith Packard
- */
-static int
-get_scaled_size (FontsetHashKey *key)
-{
-  PangoMatrix *matrix = &key->matrix;
-  double det = matrix->xx * matrix->yy - matrix->yx * matrix->xy;
-
-  if (det == 0)
-    {
-      return 0.0;
-    }
-  else
-    {
-      double x = matrix->xx;
-      double y = matrix->yx;
-      double major, minor;
-
-      major = sqrt (x*x + y*y);
-      
-      /*
-       * ignore mirroring
-       */
-      if (det < 0)
-	det = - det;
-      
-      if (major)
-	minor = det / major;
-      else 
-	minor = 0.0;
-
-      return minor * (key->size / 1024.);
-    }
-}
-
 static PangoFcPatternSet *
 pango_fc_font_map_get_patterns (PangoFontMap               *fontmap,
 				PangoContext               *context,
@@ -1079,8 +1081,10 @@ pango_fc_font_map_get_patterns (PangoFontMap               *fontmap,
 
   if (patterns == NULL)
     {
+      double scale_factor = pango_matrix_get_font_scale_factor (&key.matrix);
+      double scaled_size = key.size * scale_factor / PANGO_SCALE;
       pattern = pango_fc_make_pattern (desc, language,
-				       get_scaled_size (&key),
+				       scaled_size,
 				       pango_fc_font_map_get_resolution (fcfontmap, context));
       
       pango_fc_default_substitute (fcfontmap, context, pattern);
@@ -1103,7 +1107,7 @@ pango_fc_font_map_get_patterns (PangoFontMap               *fontmap,
 	  exit (1);
 	}
 
-      patterns = g_new (PangoFcPatternSet, 1);
+      patterns = g_slice_new (PangoFcPatternSet);
       patterns->patterns = g_new (FcPattern *, font_patterns->nfont);
       patterns->n_patterns = 0;
       patterns->fontset = NULL;
@@ -1197,7 +1201,7 @@ pango_fc_pattern_set_free (PangoFcPatternSet *patterns)
     FcPatternDestroy (patterns->patterns[i]);
 
   g_free (patterns->patterns);
-  g_free (patterns);
+  g_slice_free (PangoFcPatternSet, patterns);
 }
 
 static void
@@ -1598,6 +1602,7 @@ pango_fc_font_description_from_pattern (FcPattern *pattern, gboolean include_siz
   PangoWeight weight;
   PangoStretch stretch;
   double size;
+  PangoGravity gravity;
   
   FcChar8 *s;
   int i;
@@ -1638,6 +1643,16 @@ pango_fc_font_description_from_pattern (FcPattern *pattern, gboolean include_siz
   if (include_size && FcPatternGetDouble (pattern, FC_SIZE, 0, &size) == FcResultMatch)
     pango_font_description_set_size (desc, size * PANGO_SCALE);
 
+  if (FcPatternGetString (pattern, PANGO_FC_GRAVITY, 0, (FcChar8 **)&s) == FcResultMatch)
+    {
+      GEnumValue *value = g_enum_get_value_by_nick (get_gravity_class (), s);
+      gravity = value->value;
+    }
+  else
+    gravity = PANGO_GRAVITY_SOUTH;
+
+  pango_font_description_set_gravity (desc, gravity);
+
   return desc;
 }
 
@@ -1654,9 +1669,7 @@ make_alias_description (PangoFcFamily *fcfamily,
 
   pango_font_description_set_family (desc, fcfamily->family_name);
   pango_font_description_set_style (desc, italic ? PANGO_STYLE_ITALIC : PANGO_STYLE_NORMAL);
-  pango_font_description_set_variant (desc, PANGO_VARIANT_NORMAL);
   pango_font_description_set_weight (desc, bold ? PANGO_WEIGHT_BOLD : PANGO_WEIGHT_NORMAL);
-  pango_font_description_set_stretch (desc, PANGO_STRETCH_NORMAL);
 
   return desc;
 }
@@ -1695,6 +1708,10 @@ pango_fc_face_describe (PangoFontFace *face)
     {
       desc = pango_fc_font_description_from_pattern (result_pattern, FALSE);
       FcPatternDestroy (result_pattern);
+      /* Unset gravity.  We want gravity to be set to descriptions of fonts,
+       * but not faces.
+       */
+      pango_font_description_unset_fields (desc, PANGO_FONT_MASK_GRAVITY);
     }
 
   FcPatternDestroy (match_pattern);
@@ -1825,7 +1842,7 @@ pango_fc_face_get_type (void)
 
   if (!object_type)
     {
-      static const GTypeInfo object_info =
+      const GTypeInfo object_info =
       {
         sizeof (PangoFontFaceClass),
         (GBaseInitFunc) NULL,
@@ -1836,10 +1853,11 @@ pango_fc_face_get_type (void)
         sizeof (PangoFcFace),
         0,              /* n_preallocs */
         (GInstanceInitFunc) NULL,
+	NULL            /* value_table */
       };
       
       object_type = g_type_register_static (PANGO_TYPE_FONT_FACE,
-                                            "PangoFcFace",
+                                            I_("PangoFcFace"),
                                             &object_info, 0);
     }
   
@@ -1952,7 +1970,7 @@ pango_fc_family_get_type (void)
 
   if (!object_type)
     {
-      static const GTypeInfo object_info =
+      const GTypeInfo object_info =
       {
         sizeof (PangoFontFamilyClass),
         (GBaseInitFunc) NULL,
@@ -1963,10 +1981,11 @@ pango_fc_family_get_type (void)
         sizeof (PangoFcFamily),
         0,              /* n_preallocs */
         (GInstanceInitFunc) pango_fc_family_init,
+	NULL            /* value_table */
       };
       
       object_type = g_type_register_static (PANGO_TYPE_FONT_FAMILY,
-                                            "PangoFcFamily",
+                                            I_("PangoFcFamily"),
                                             &object_info, 0);
     }
   
